@@ -72,21 +72,18 @@ myhtml_status_t myhtml_hread_sem_create(mythread_t *mythread, mythread_context_t
 {
     ctx->sem_name = calloc(1024, sizeof(wchar_t));
     
-    char tmp[1024] = {0};
-    sprintf_s(tmp, 1024, "Global/%s%zus%zup%zu", MyTHREAD_SEM_NAME, prefix_id, ctx->id, (size_t)mythread);
+    sprintf(ctx->sem_name, "Global/%s%zus%zup%zu", MyTHREAD_SEM_NAME, prefix_id, ctx->id, (size_t)mythread);
     
-    size_t retval = 0;
-    mbstowcs_s(&retval, ctx->sem_name, 1024, tmp, strlen(tmp) + 1);
-    
-    ctx->sem_name_size = wcslen(ctx->sem_name);
+    ctx->sem_name_size = strlen(ctx->sem_name);
     
     ctx->sem = CreateSemaphore(NULL,           // default security attributes
                                0,              // initial count
-                               0,              // maximum count
-                               ctx->sem_name); // unnamed semaphore
+                               1,              // maximum count
+                               ctx->sem_name); // named semaphore
     
     if (ctx->sem == NULL) {
         free(ctx->sem_name);
+        ctx->sem_name = NULL;
         
         mythread->sys_last_error = GetLastError();
         return MyHTML_STATUS_THREAD_ERROR_SEM_CREATE;
@@ -253,12 +250,6 @@ mythread_t * mythread_create(void)
 
 myhtml_status_t mythread_init(mythread_t *mythread, const char *sem_prefix, size_t thread_count, size_t queue_size)
 {
-    myhtml_status_t status;
-    mythread->queue = mythread_queue_create(4096, &status);
-    
-    if(mythread->queue == NULL)
-        return status;
-    
     return MyHTML_STATUS_OK;
 }
 
@@ -268,8 +259,8 @@ myhtml_status_t mythread_init(mythread_t *mythread, const char *sem_prefix, size
 {
     mythread->batch_count    = 0;
     mythread->batch_first_id = 0;
-    mythread->stream_opt     = MyTHREAD_OPT_WAIT;
-    mythread->batch_opt      = MyTHREAD_OPT_WAIT;
+    mythread->stream_opt     = MyTHREAD_OPT_STOP;
+    mythread->batch_opt      = MyTHREAD_OPT_STOP;
     
     if(thread_count)
     {
@@ -296,9 +287,9 @@ myhtml_status_t mythread_init(mythread_t *mythread, const char *sem_prefix, size
     }
     
     myhtml_status_t status;
-    mythread->queue = mythread_queue_create(4096, &status);
+    mythread->queue_list = mythread_queue_list_create(mythread, 1024, &status);
     
-    if(mythread->queue == NULL)
+    if(mythread->queue_list == NULL)
         return status;
     
     if(sem_prefix)
@@ -325,22 +316,6 @@ myhtml_status_t mythread_init(mythread_t *mythread, const char *sem_prefix, size
 void mythread_clean(mythread_t *mythread)
 {
     mythread->sys_last_error = 0;
-    
-    if(mythread->queue)
-        mythread_queue_clean(mythread->queue);
-    
-#ifndef MyHTML_BUILD_WITHOUT_THREADS
-    
-    size_t idx;
-    for (idx = mythread->pth_list_root; idx < mythread->pth_list_length; idx++) {
-        mythread->pth_list[idx].data.use = 0;
-    }
-    
-    for (idx = 0; idx < mythread->batch_count; idx++) {
-        mythread->pth_list[( mythread->batch_first_id + idx )].data.use = idx;
-    }
-    
-#endif /* MyHTML_BUILD_WITHOUT_THREADS */
 }
 
 mythread_t * mythread_destroy(mythread_t *mythread, bool self_destroy)
@@ -356,7 +331,7 @@ mythread_t * mythread_destroy(mythread_t *mythread, bool self_destroy)
         mythread_resume_all(mythread);
         mythread_stream_quit_all(mythread);
         mythread_batch_quit_all(mythread);
-        mythread_wait_all(mythread);
+        mythread_wait_all_for_done(mythread);
         
         for (size_t i = mythread->pth_list_root; i < mythread->pth_list_length; i++)
         {
@@ -371,6 +346,10 @@ mythread_t * mythread_destroy(mythread_t *mythread, bool self_destroy)
         mythread->pth_list = NULL;
     }
     
+    if(mythread->queue_list) {
+        free(mythread->queue_list);
+    }
+    
     if(mythread->sem_prefix) {
         free(mythread->sem_prefix);
         
@@ -380,9 +359,6 @@ mythread_t * mythread_destroy(mythread_t *mythread, bool self_destroy)
     
 #endif /* MyHTML_BUILD_WITHOUT_THREADS */
     
-    if(mythread->queue)
-        mythread_queue_destroy(mythread->queue);
-    
     if(self_destroy) {
         free(mythread);
         return NULL;
@@ -412,9 +388,8 @@ mythread_id_t _myhread_create_stream_raw(mythread_t *mythread, mythread_f func,
     thr->data.mythread = mythread;
     thr->data.func     = func;
     thr->data.id       = mythread->pth_list_length;
-    thr->data.use      = 0;
-    thr->data.qnode    = NULL;
     thr->data.t_count  = total_count;
+    thr->data.opt      = MyTHREAD_OPT_STOP;
     
     myhtml_status_t m_status = myhtml_hread_sem_create(mythread, &thr->data, 0);
     
@@ -460,8 +435,8 @@ mythread_id_t myhread_create_batch(mythread_t *mythread, mythread_f func, myhtml
     
     bool init_first = false;
     
-    for (size_t i = 0; i < count; i++) {
-        
+    for (size_t i = 0; i < count; i++)
+    {
         mythread_id_t curr_id = _myhread_create_stream_raw(mythread, func, mythread_function_batch, status, count);
         
         if(init_first == false) {
@@ -489,9 +464,6 @@ mythread_id_t myhread_create_batch(mythread_t *mythread, mythread_f func, myhtml
             
             break;
         }
-        else {
-            mythread->pth_list[curr_id].data.use = i;
-        }
     }
     
     return mythread->batch_first_id;
@@ -500,6 +472,151 @@ mythread_id_t myhread_create_batch(mythread_t *mythread, mythread_f func, myhtml
 #endif /* MyHTML_BUILD_WITHOUT_THREADS */
 
 // mythread queue functions
+#ifndef MyHTML_BUILD_WITHOUT_THREADS
+mythread_queue_list_t * mythread_queue_list_create(mythread_t *mythread, size_t size, myhtml_status_t *status)
+{
+    if(status)
+        *status = MyHTML_STATUS_OK;
+    
+    if(size < 1024)
+        size = 1024;
+    
+    mythread_queue_list_t* queue_list = (mythread_queue_list_t*)mycalloc(1, sizeof(mythread_queue_list_t));
+    
+    if(queue_list == NULL) {
+        if(status)
+            *status = MyHTML_STATUS_THREAD_ERROR_QUEUE_MALLOC;
+        return NULL;
+    }
+    
+    return queue_list;
+}
+
+mythread_queue_list_entry_t * mythread_queue_list_entry_push(mythread_t *mythread, mythread_queue_t *queue, myhtml_status_t *status)
+{
+    mythread_queue_list_t *queue_list = mythread->queue_list;
+    
+    if(status)
+        *status = MyHTML_STATUS_OK;
+    
+    mythread_queue_list_entry_t* entry = (mythread_queue_list_entry_t*)mycalloc(1, sizeof(mythread_queue_list_entry_t));
+    
+    if(entry == NULL) {
+        if(status)
+            *status = MyHTML_STATUS_THREAD_ERROR_QUEUE_MALLOC;
+        return NULL;
+    }
+    
+    entry->thread_param = (mythread_queue_thread_param_t*)mycalloc(mythread->pth_list_size, sizeof(mythread_queue_thread_param_t));
+    
+    if(entry == NULL) {
+        free(entry);
+        
+        if(status)
+            *status = MyHTML_STATUS_THREAD_ERROR_QUEUE_MALLOC;
+        return NULL;
+    }
+    
+    entry->queue = queue;
+    
+    if(mythread->stream_opt == MyTHREAD_OPT_UNDEF) {
+        mythread_suspend_all(mythread);
+    }
+    else if(mythread->stream_opt == MyTHREAD_OPT_STOP) {
+        mythread_stop_all(mythread);
+    }
+    
+    if(queue_list->first) {
+        queue_list->last->next = entry;
+        entry->prev = queue_list->last;
+        
+        queue_list->last = entry;
+    }
+    else {
+        queue_list->first = entry;
+        queue_list->last = entry;
+    }
+    
+    queue_list->count++;
+    
+    if(mythread->stream_opt != MyTHREAD_OPT_STOP)
+        mythread_resume_all(mythread);
+    
+    return entry;
+}
+
+mythread_queue_list_entry_t * mythread_queue_list_entry_delete(mythread_t *mythread, mythread_queue_list_entry_t *entry, bool destroy_queue)
+{
+    mythread_queue_list_t *queue_list = mythread->queue_list;
+    
+    mythread_queue_list_entry_t *next = entry->next;
+    mythread_queue_list_entry_t *prev = entry->prev;
+    
+    if(mythread->stream_opt == MyTHREAD_OPT_UNDEF) {
+        mythread_suspend_all(mythread);
+    }
+    else if(mythread->stream_opt == MyTHREAD_OPT_STOP) {
+        mythread_stop_all(mythread);
+    }
+    
+    if(prev)
+        prev->next = next;
+    
+    if(next)
+        next->prev = prev;
+    
+    if(queue_list->first == entry)
+        queue_list->first = next;
+    
+    if(queue_list->last == entry)
+        queue_list->last = prev;
+    
+    if(mythread->stream_opt != MyTHREAD_OPT_STOP)
+        mythread_resume_all(mythread);
+    
+    if(destroy_queue && entry->queue)
+        mythread_queue_destroy(entry->queue);
+    
+    if(entry->thread_param)
+        free(entry->thread_param);
+    
+    free(entry);
+    
+    queue_list->count--;
+    
+    return NULL;
+}
+
+void mythread_queue_list_entry_clean(mythread_t *mythread, mythread_queue_list_entry_t *entry)
+{
+    if(entry == NULL)
+        return;
+    
+    mythread_queue_clean(entry->queue);
+    
+    size_t idx;
+    for (idx = mythread->pth_list_root; idx < mythread->pth_list_size; idx++) {
+        entry->thread_param[idx].use = 0;
+    }
+}
+
+void mythread_queue_list_entry_wait_for_done(mythread_t *mythread, mythread_queue_list_entry_t *entry)
+{
+    if(entry == NULL)
+        return;
+    
+    size_t idx;
+    const struct timespec tomeout = {0, 10000};
+    
+    for (idx = mythread->pth_list_root; idx < mythread->pth_list_size; idx++) {
+        while(entry->thread_param[idx].use < entry->queue->nodes_uses) {
+            myhtml_thread_nanosleep(&tomeout);
+        }
+    }
+}
+
+#endif /* MyHTML_BUILD_WITHOUT_THREADS */
+
 mythread_queue_t * mythread_queue_create(size_t size, myhtml_status_t *status)
 {
     if(status)
@@ -508,7 +625,7 @@ mythread_queue_t * mythread_queue_create(size_t size, myhtml_status_t *status)
     if(size < 4096)
         size = 4096;
     
-    mythread_queue_t* queue = (mythread_queue_t*)malloc(sizeof(mythread_queue_t));
+    mythread_queue_t* queue = (mythread_queue_t*)mymalloc(sizeof(mythread_queue_t));
     
     if(queue == NULL) {
         if(status)
@@ -518,7 +635,7 @@ mythread_queue_t * mythread_queue_create(size_t size, myhtml_status_t *status)
     
     queue->nodes_pos_size = 512;
     queue->nodes_size     = size;
-    queue->nodes          = (mythread_queue_node_t**)calloc(queue->nodes_pos_size, sizeof(mythread_queue_node_t*));
+    queue->nodes          = (mythread_queue_node_t**)mycalloc(queue->nodes_pos_size, sizeof(mythread_queue_node_t*));
     
     if(queue->nodes == NULL) {
         free(queue);
@@ -530,7 +647,7 @@ mythread_queue_t * mythread_queue_create(size_t size, myhtml_status_t *status)
     
     mythread_queue_clean(queue);
     
-    queue->nodes[queue->nodes_pos] = (mythread_queue_node_t*)malloc(sizeof(mythread_queue_node_t) * queue->nodes_size);
+    queue->nodes[queue->nodes_pos] = (mythread_queue_node_t*)mymalloc(sizeof(mythread_queue_node_t) * queue->nodes_size);
     
     if(queue->nodes[queue->nodes_pos] == NULL) {
         free(queue->nodes);
@@ -578,17 +695,9 @@ void mythread_queue_node_clean(mythread_queue_node_t* qnode)
     memset(qnode, 0, sizeof(mythread_queue_node_t));
 }
 
-mythread_queue_node_t * mythread_queue_get_prev_node(mythread_queue_t* queue)
+mythread_queue_node_t * mythread_queue_get_prev_node(mythread_queue_node_t* qnode)
 {
-    if(queue->nodes_length == 0)
-    {
-        if(queue->nodes_pos == 0)
-            return NULL;
-        
-        return &queue->nodes[(queue->nodes_pos - 1)][(queue->nodes_size - 1)];
-    }
-    
-    return &queue->nodes[queue->nodes_pos][(queue->nodes_length - 1)];
+    return qnode->prev;
 }
 
 mythread_queue_node_t * mythread_queue_get_current_node(mythread_queue_t* queue)
@@ -606,7 +715,7 @@ size_t mythread_queue_count_used_node(mythread_queue_t* queue)
     return queue->nodes_uses;
 }
 
-mythread_queue_node_t * mythread_queue_node_malloc(mythread_queue_t* queue, const char* text, size_t begin, myhtml_status_t *status)
+mythread_queue_node_t * mythread_queue_node_malloc(mythread_t *mythread, mythread_queue_t* queue, const char* text, size_t begin, myhtml_status_t *status)
 {
     queue->nodes_length++;
     
@@ -616,6 +725,8 @@ mythread_queue_node_t * mythread_queue_node_malloc(mythread_queue_t* queue, cons
         
         if(queue->nodes_pos >= queue->nodes_pos_size)
         {
+            mythread_wait_all_for_done(mythread);
+            
             queue->nodes_pos_size <<= 1;
             mythread_queue_node_t** tmp = realloc(queue->nodes, sizeof(mythread_queue_node_t*) * queue->nodes_pos_size);
             
@@ -656,16 +767,104 @@ mythread_queue_node_t * mythread_queue_node_malloc(mythread_queue_t* queue, cons
     return qnode;
 }
 
+mythread_queue_node_t * mythread_queue_node_malloc_limit(mythread_t *mythread, mythread_queue_t* queue, const char* text, size_t begin, size_t limit, myhtml_status_t *status)
+{
+    queue->nodes_length++;
+    
+    if(queue->nodes_uses >= limit) {
+        mythread_wait_all_for_done(mythread);
+        
+        queue->nodes_length = 0;
+        queue->nodes_pos    = 0;
+        queue->nodes_root   = 0;
+        queue->nodes_uses   = 0;
+    }
+    else if(queue->nodes_length >= queue->nodes_size)
+    {
+        queue->nodes_pos++;
+        
+        if(queue->nodes_pos >= queue->nodes_pos_size)
+        {
+            mythread_wait_all_for_done(mythread);
+            
+            queue->nodes_pos_size <<= 1;
+            mythread_queue_node_t** tmp = realloc(queue->nodes, sizeof(mythread_queue_node_t*) * queue->nodes_pos_size);
+            
+            if(tmp) {
+                memset(&tmp[queue->nodes_pos], 0, sizeof(mythread_queue_node_t*) * (queue->nodes_pos_size - queue->nodes_pos));
+                
+                queue->nodes = tmp;
+            }
+            else {
+                if(status)
+                    *status = MyHTML_STATUS_THREAD_ERROR_QUEUE_NODES_MALLOC;
+                
+                return NULL;
+            }
+        }
+        
+        if(queue->nodes[queue->nodes_pos] == NULL) {
+            queue->nodes[queue->nodes_pos] = (mythread_queue_node_t*)malloc(sizeof(mythread_queue_node_t) * queue->nodes_size);
+            
+            if(queue->nodes[queue->nodes_pos] == NULL) {
+                if(status)
+                    *status = MyHTML_STATUS_THREAD_ERROR_QUEUE_NODE_MALLOC;
+                
+                return NULL;
+            }
+        }
+        
+        queue->nodes_length = 0;
+    }
+    
+    queue->nodes_uses++;
+    
+    mythread_queue_node_t *qnode = &queue->nodes[queue->nodes_pos][queue->nodes_length];
+    
+    qnode->text  = text;
+    qnode->begin = begin;
+    
+    return qnode;
+}
+
+#ifndef MyHTML_BUILD_WITHOUT_THREADS
+
+mythread_queue_node_t * mythread_queue_node_malloc_round(mythread_t *mythread, mythread_queue_list_entry_t *entry,
+                                                              const char* text, size_t begin, myhtml_status_t *status)
+{
+    mythread_queue_t* queue = entry->queue;
+    
+    queue->nodes_length++;
+    
+    if(queue->nodes_length >= queue->nodes_size) {
+        queue->nodes_uses++;
+        
+        mythread_queue_list_entry_wait_for_done(mythread, entry);
+        mythread_queue_list_entry_clean(mythread, entry);
+    }
+    else
+        queue->nodes_uses++;
+    
+    mythread_queue_node_t *qnode = &queue->nodes[queue->nodes_pos][queue->nodes_length];
+    
+    qnode->text  = text;
+    qnode->begin = begin;
+    
+    return qnode;
+}
+
+#endif /* MyHTML_BUILD_WITHOUT_THREADS */
+
 #ifdef MyHTML_BUILD_WITHOUT_THREADS
 
 void mythread_stream_quit_all(mythread_t *mythread) {}
 void mythread_batch_quit_all(mythread_t *mythread) {}
-void mythread_stream_pause_all(mythread_t *mythread) {}
-void mythread_batch_pause_all(mythread_t *mythread) {}
-void mythread_resume_all(mythread_t *mythread) {}
-void mythread_wait_all(mythread_t *mythread) {}
-void mythread_function_batch(void *arg) {}
-void mythread_function_stream(void *arg) {}
+void mythread_stream_stop_all(mythread_t *mythread) {};
+void mythread_batch_stop_all(mythread_t *mythread) {};
+void mythread_stop_all(mythread_t *mythread) {};
+void mythread_resume_all(mythread_t *mythread) {};
+void mythread_wait_all_for_done(mythread_t *mythread) {};
+void mythread_suspend_all(mythread_t *mythread) {};
 
 #else /* MyHTML_BUILD_WITHOUT_THREADS */
 
@@ -680,14 +879,40 @@ void mythread_batch_quit_all(mythread_t *mythread)
     mythread->batch_opt = MyTHREAD_OPT_QUIT;
 }
 
-void mythread_stream_pause_all(mythread_t *mythread)
+void mythread_stream_stop_all(mythread_t *mythread)
 {
-    mythread->stream_opt = MyTHREAD_OPT_WAIT;
+    if(mythread->stream_opt != MyTHREAD_OPT_STOP)
+        mythread->stream_opt = MyTHREAD_OPT_STOP;
+    
+    size_t idx;
+    const struct timespec tomeout = {0, 10000};
+    
+    for (idx = mythread->pth_list_root; idx < mythread->batch_first_id; idx++) {
+        while(mythread->pth_list[idx].data.opt != MyTHREAD_OPT_STOP) {
+            myhtml_thread_nanosleep(&tomeout);
+        }
+    }
 }
 
-void mythread_batch_pause_all(mythread_t *mythread)
+void mythread_batch_stop_all(mythread_t *mythread)
 {
-    mythread->batch_opt = MyTHREAD_OPT_WAIT;
+    if(mythread->batch_opt != MyTHREAD_OPT_STOP)
+        mythread->batch_opt = MyTHREAD_OPT_STOP;
+    
+    size_t idx;
+    const struct timespec tomeout = {0, 10000};
+    
+    for (idx = mythread->batch_first_id; idx < (mythread->batch_first_id + mythread->batch_count); idx++) {
+        while(mythread->pth_list[idx].data.opt != MyTHREAD_OPT_STOP) {
+            myhtml_thread_nanosleep(&tomeout);
+        }
+    }
+}
+
+void mythread_stop_all(mythread_t *mythread)
+{
+    mythread_stream_stop_all(mythread);
+    mythread_batch_stop_all(mythread);
 }
 
 void mythread_resume_all(mythread_t *mythread)
@@ -696,64 +921,159 @@ void mythread_resume_all(mythread_t *mythread)
        mythread->batch_opt  == MyTHREAD_OPT_UNDEF)
         return;
     
-    mythread->stream_opt = MyTHREAD_OPT_UNDEF;
-    mythread->batch_opt  = MyTHREAD_OPT_UNDEF;
-    
-    for (size_t idx = mythread->pth_list_root; idx < mythread->pth_list_length; idx++) {
-        myhtml_hread_sem_post(mythread, &mythread->pth_list[idx].data);
+    if(mythread->stream_opt == MyTHREAD_OPT_WAIT ||
+       mythread->batch_opt == MyTHREAD_OPT_WAIT)
+    {
+        mythread->stream_opt = MyTHREAD_OPT_UNDEF;
+        mythread->batch_opt  = MyTHREAD_OPT_UNDEF;
+    }
+    else {
+        mythread->stream_opt = MyTHREAD_OPT_UNDEF;
+        mythread->batch_opt  = MyTHREAD_OPT_UNDEF;
+        
+        for (size_t idx = mythread->pth_list_root; idx < mythread->pth_list_size; idx++) {
+            myhtml_hread_sem_post(mythread, &mythread->pth_list[idx].data);
+        }
     }
 }
 
-void mythread_wait_all(mythread_t *mythread)
+void mythread_wait_all_for_done(mythread_t *mythread)
 {
     const struct timespec tomeout = {0, 10000};
     
-    for (size_t idx = mythread->pth_list_root; idx < mythread->pth_list_length; idx++) {
-        while(mythread->pth_list[idx].data.use < mythread->queue->nodes_uses) {
+    mythread_queue_list_t *queue_list = mythread->queue_list;
+    mythread_queue_list_entry_t *entry = queue_list->first;
+    
+    while(entry)
+    {
+        for (size_t idx = mythread->pth_list_root; idx < mythread->pth_list_size; idx++) {
+            while(entry->thread_param[idx].use < entry->queue->nodes_uses) {
+                myhtml_thread_nanosleep(&tomeout);
+            }
+        }
+        
+        entry = entry->next;
+    }
+}
+
+void mythread_suspend_all(mythread_t *mythread)
+{
+    if(mythread->stream_opt != MyTHREAD_OPT_WAIT)
+        mythread->stream_opt = MyTHREAD_OPT_WAIT;
+    
+    if(mythread->batch_opt != MyTHREAD_OPT_WAIT)
+        mythread->batch_opt  = MyTHREAD_OPT_WAIT;
+    
+    const struct timespec tomeout = {0, 10000};
+    
+    for (size_t idx = mythread->pth_list_root; idx < mythread->pth_list_size; idx++) {
+        while(mythread->pth_list[idx].data.opt != MyTHREAD_OPT_WAIT) {
             myhtml_thread_nanosleep(&tomeout);
         }
     }
 }
 
+bool mythread_function_see_for_all_done(mythread_queue_list_t *queue_list, size_t thread_id)
+{
+    size_t done_count = 0;
+    
+    mythread_queue_list_entry_t *entry = queue_list->first;
+    while(entry)
+    {
+        if(entry->thread_param[ thread_id ].use >= entry->queue->nodes_uses) {
+            done_count++;
+            entry = entry->next;
+        }
+        else
+            break;
+    }
+    
+    return done_count == queue_list->count;
+}
+
+bool mythread_function_see_opt(mythread_context_t *ctx, volatile mythread_thread_opt_t opt, size_t done_count, const struct timespec *timeout)
+{
+    mythread_t * mythread = ctx->mythread;
+    mythread_queue_list_t *queue_list = mythread->queue_list;
+    
+    if(done_count != queue_list->count)
+        return false;
+    
+    if(opt & MyTHREAD_OPT_STOP)
+    {
+        if(mythread_function_see_for_all_done(queue_list, ctx->id))
+        {
+            ctx->opt = MyTHREAD_OPT_STOP;
+            myhtml_hread_sem_wait(mythread, ctx);
+            ctx->opt = MyTHREAD_OPT_UNDEF;
+            
+            return false;
+        }
+    }
+    else if(opt & MyTHREAD_OPT_QUIT)
+    {
+        if(mythread_function_see_for_all_done(queue_list, ctx->id))
+        {
+            myhtml_hread_sem_close(mythread, ctx);
+            ctx->opt = MyTHREAD_OPT_QUIT;
+            return true;
+        }
+    }
+    
+    myhtml_thread_nanosleep(timeout);
+    
+    return false;
+}
+
 void mythread_function_batch(void *arg)
 {
     mythread_context_t *ctx = (mythread_context_t*)arg;
     mythread_t * mythread = ctx->mythread;
-    mythread_queue_t *queue = mythread->queue;
+    mythread_queue_list_t *queue_list = mythread->queue_list;
     
-    const struct timespec tomeout = {0, 10000};
+    const struct timespec timeout = {0, 10000};
     
     myhtml_hread_sem_wait(mythread, ctx);
     
     do {
-        while (ctx->use >= queue->nodes_uses) {
-            if(mythread->batch_opt & MyTHREAD_OPT_WAIT) {
-                if(ctx->use >= queue->nodes_uses) {
-                    ctx->opt = MyTHREAD_OPT_WAIT;
-                    myhtml_hread_sem_wait(mythread, ctx);
-                    ctx->opt = MyTHREAD_OPT_UNDEF;
-                }
-            }
-            else if(mythread->batch_opt & MyTHREAD_OPT_QUIT) {
-                if(ctx->use >= queue->nodes_uses) {
-                    myhtml_hread_sem_close(mythread, ctx);
-                    ctx->opt = MyTHREAD_OPT_QUIT;
-                    return;
-                }
+        if(mythread->batch_opt & MyTHREAD_OPT_WAIT) {
+            ctx->opt = MyTHREAD_OPT_WAIT;
+            
+            while (mythread->batch_opt & MyTHREAD_OPT_WAIT) {
+                myhtml_thread_nanosleep(&timeout);
             }
             
-            myhtml_thread_nanosleep(&tomeout);
+            ctx->opt = MyTHREAD_OPT_UNDEF;
         }
         
-        size_t pos = ctx->use / queue->nodes_size;
-        size_t len = ctx->use % queue->nodes_size;
+        mythread_queue_list_entry_t *entry = queue_list->first;
+        size_t done_count = 0;
         
-        mythread_queue_node_t *qnode = &queue->nodes[pos][len];
+        while(entry)
+        {
+            mythread_queue_thread_param_t *thread_param = &entry->thread_param[ ctx->id ];
+            
+            if(thread_param->use < entry->queue->nodes_uses)
+            {
+                size_t pos = thread_param->use / entry->queue->nodes_size;
+                size_t len = thread_param->use % entry->queue->nodes_size;
+                
+                mythread_queue_node_t *qnode = &entry->queue->nodes[pos][len];
+                
+                if((qnode->tree->flags & MyHTML_TREE_FLAGS_SINGLE_MODE) == 0)
+                    ctx->func(ctx->id, qnode);
+                
+                thread_param->use += ctx->t_count;
+            }
+            else
+                done_count++;
+            
+            entry = entry->next;
+        }
         
-        if((qnode->tree->flags & MyHTML_TREE_FLAGS_SINGLE_MODE) == 0)
-            ctx->func(ctx->id, qnode);
-        
-        ctx->use += ctx->t_count;
+        if(done_count == queue_list->count &&
+           mythread_function_see_opt(ctx, mythread->batch_opt, done_count, &timeout))
+            break;
     }
     while (1);
 }
@@ -762,41 +1082,50 @@ void mythread_function_stream(void *arg)
 {
     mythread_context_t *ctx = (mythread_context_t*)arg;
     mythread_t * mythread = ctx->mythread;
-    mythread_queue_t *queue = mythread->queue;
-    
-    const struct timespec tomeout = {0, 10000};
+    mythread_queue_list_t *queue_list = mythread->queue_list;
     
+    const struct timespec timeout = {0, 10000};
     myhtml_hread_sem_wait(mythread, ctx);
     
     do {
-        while (ctx->use >= queue->nodes_uses) {
-            if(mythread->stream_opt & MyTHREAD_OPT_WAIT) {
-                if(ctx->use >= queue->nodes_uses) {
-                    ctx->opt = MyTHREAD_OPT_WAIT;
-                    myhtml_hread_sem_wait(mythread, ctx);
-                    ctx->opt = MyTHREAD_OPT_UNDEF;
-                }
-            }
-            else if(mythread->stream_opt & MyTHREAD_OPT_QUIT) {
-                if(ctx->use >= queue->nodes_uses) {
-                    myhtml_hread_sem_close(mythread, ctx);
-                    ctx->opt = MyTHREAD_OPT_QUIT;
-                    return;
-                }
+        if(mythread->stream_opt & MyTHREAD_OPT_WAIT) {
+            ctx->opt = MyTHREAD_OPT_WAIT;
+            
+            while (mythread->stream_opt & MyTHREAD_OPT_WAIT) {
+                myhtml_thread_nanosleep(&timeout);
             }
             
-            myhtml_thread_nanosleep(&tomeout);
+            ctx->opt = MyTHREAD_OPT_UNDEF;
         }
         
-        size_t pos = ctx->use / queue->nodes_size;
-        size_t len = ctx->use % queue->nodes_size;
+        mythread_queue_list_entry_t *entry = queue_list->first;
+        size_t done_count = 0;
         
-        mythread_queue_node_t *qnode = &queue->nodes[pos][len];
+        while(entry)
+        {
+            mythread_queue_thread_param_t *thread_param = &entry->thread_param[ ctx->id ];
+            
+            if(thread_param->use < entry->queue->nodes_uses)
+            {
+                size_t pos = thread_param->use / entry->queue->nodes_size;
+                size_t len = thread_param->use % entry->queue->nodes_size;
+                
+                mythread_queue_node_t *qnode = &entry->queue->nodes[pos][len];
+                
+                if((qnode->tree->flags & MyHTML_TREE_FLAGS_SINGLE_MODE) == 0)
+                    ctx->func(ctx->id, qnode);
+                
+                thread_param->use++;
+            }
+            else
+                done_count++;
+            
+            entry = entry->next;
+        }
         
-        if((qnode->tree->flags & MyHTML_TREE_FLAGS_SINGLE_MODE) == 0)
-            ctx->func(ctx->id, qnode);
-        
-        ctx->use++;
+        if(done_count == queue_list->count &&
+           mythread_function_see_opt(ctx, mythread->stream_opt, done_count, &timeout))
+            break;
     }
     while (1);
 }
